# Survival Analysis
# Split Population
# log logitic
# left truncation
# no fraility

llik.log.logistic.left <- function(out,left,Time,Event){
  shape <- out$shape
  scale <- out$scale
  split <- out$split                       
  log.density.s <- log((1-split)+(split*((1+((scale*left)^(1/shape)))/(1+((scale*Time)^(1/shape))))))
  log.density.t <- log(split)-log(shape)+(((1/shape)-1)*log(Time))+((1/shape)*log(scale))-(2*log(1+((scale*Time)^(1/shape))))+log(1+((scale*left)^(1/shape)))
  llik <- sum((Event*log.density.t) + ((1-Event)*log.density.s))
  return(llik)
}

llik <- function(para,left,Time,Event,Xt1,Xs1){
  # extract parameters
  kt1 <- ncol(Xt1)
  ks1 <- ncol(Xs1)
  beta <- para[1:kt1]
  gamma <- para[(kt1+1):(kt1+ks1)]
  theta1 <- Xt1%*%beta
  theta2 <- Xs1%*%gamma
  shape <- exp(para[kt1+ks1+1]) # shape parameter
  scale <- exp(-theta1) # scale parameter
  split <- logit(theta2) # the probability to be at risk
  out <- list()
  out$shape <- shape
  out$scale <- scale
  out$split <- split
  llik <- llik.log.logistic.left(out,left,Time,Event)
  return(llik)
}

main.left <- function(
 ## arguments
 formula.t,# T ~ X.t
 formula.s,# S ~ X.s
 left # left truncation
 ){
## extract data
# time component
  model.t <- model.frame(formula.t)
  Time <- model.extract(model.t, "response")
  Xt1 <- as.matrix(model.matrix(formula.t))
  Xt1names <- dimnames(Xt1)[[2]]
  obs <- length(Time)
  kt1 <- ncol(Xt1)
# split component 
  model.s <- model.frame(formula.s)
  Event <- model.extract(model.s, "response")
  Xs1 <- as.matrix(model.matrix(formula.s))
  Xs1names <- dimnames(Xs1)[[2]]
  ks1 <- ncol(Xs1)
## initial values
  temp <- survreg(Surv(Time,Event) ~  Xt1[,2:kt1],dist='weibull')
  # temp <- coxph(Surv(left,Time,Event) ~  Xt1[,2:kt1]) # no constant
  beta.init <- temp$coef
  log.shape.init <- 0 # temp$icoef[2]
  gamma.init <- glm(Event ~ Xs1[,2:ks1],family=binomial)$coef
  initial <- c(beta.init,gamma.init,log.shape.init)  
## optimzation
  res <- optim(initial, llik, method = "BFGS", hessian = TRUE, control = list(fnscale = -1,ndeps=rep(1e-10,length(initial))),
            left=left,Time=Time,Event=Event,Xt1=Xt1,Xs1=Xs1)
## output
  names(res$par) <- c(Xt1names,Xs1names,"log.shape")
  return(res)
}

genoud.left <- function(
 ## arguments
 formula.t,# T ~ X.t
 formula.s,# S ~ X.s
 left # left truncation
 ){
## extract data
# time component
  model.t <- model.frame(formula.t)
  Time <- model.extract(model.t, "response")
  Xt1 <- as.matrix(model.matrix(formula.t))
  Xt1names <- dimnames(Xt1)[[2]]
  obs <- length(Time)
  kt1 <- ncol(Xt1)
# split component 
  model.s <- model.frame(formula.s)
  Event <- model.extract(model.s, "response")
  Xs1 <- as.matrix(model.matrix(formula.s))
  Xs1names <- dimnames(Xs1)[[2]]
  ks1 <- ncol(Xs1)
## initial values
  temp <- survreg(Surv(Time,Event) ~  Xt1[,2:kt1],dist='weibull')
  # temp <- coxph(Surv(left,Time,Event) ~  Xt1[,2:kt1]) # no constant
  beta.init <- temp$coef
  log.shape.init <- 0 # temp$icoef[2]
  gamma.init <- glm(Event ~ Xs1[,2:ks1],family=binomial)$coef
  initial <- c(beta.init,gamma.init,log.shape.init)  
## optimzation
  nvars <- length(initial)
  res <- genoud(llik, nvars, hessian = TRUE, solution.tolerance=10^(-10),
                left=left,Time=Time,Event=Event,Xt1=Xt1,Xs1=Xs1) 
## output
  names(res$par) <- c(Xt1names,Xs1names,"log.shape")
  return(res)
}


dg.logistic.left <- function(obs,beta,gamma,log.shape){
  kt <- length(beta)-1
  ks <- length(gamma)-1
  Xt1 <- cbind(rep(1,obs),matrix(rnorm(obs*kt),obs,kt))
  Xs1 <- cbind(rep(1,obs),matrix(rnorm(obs*ks),obs,ks))
  theta1 <- Xt1%*%beta
  theta2 <- Xs1%*%gamma
  shape <- exp(log.shape) # shape parameter
  scale <- exp(-theta1) # scale parameter
  split <- logit(theta2) # the probability to be at risk
# latent event time
  Time <- (odds(runif(obs))^shape)/scale
  censor.time <- rep(40,obs)
  # censor.time <- Time[sample(1:obs,obs,replace=T)]*2 # *1.5?
  begin <- apply(cbind(Time,Time[sample(1:obs,obs,replace=T)],censor.time),1,min)
  #begin[Time<begin] <- Time[Time<begin]
  Event <- as.integer((Time <= censor.time) & (runif(obs) <= split))
  Time[Event==0] <- censor.time[Event==0] 
  out <- list()
  out$Time <- Time
  out$Event <- Event
  out$begin <- begin
  out$Xt <- Xt1[,2:(kt+1)]
  out$Xs <- Xs1[,2:(ks+1)]
  out
}





